@@ -5,7 +5,7 @@ import sys
 import json
 from zipfile import ZipFile
 from io import TextIOWrapper
-from typing import Optional, Union
+from typing import Optional, Union, Callable
 from threading import Thread
 
 from flask import request
@@ -14,12 +14,15 @@ from dotenv import load_dotenv
 from .languages import LanguageParser
 from .constants import TEMPLATES_FOLDER, STARTER_FILES, NOT_FOUND_FILE, \
         CONFIG_FILE, STARTER_CONFIG_FILE, IS_RUNNING, PROJECTS_DIR, \
-        CURRENT_PROJECT_DIR, DOT_RUNIT_IGNORE
+        CURRENT_PROJECT_DIR, DOT_RUNIT_IGNORE, INSTALL_MODULE_LATER_MESSAGE
 
 load_dotenv()
 class RunIt:
+    DOCKER = False
+    KUBERNETES = False
+
     def __init__(self, name, _id="", version="0.0.1", description="", homepage="",
-    language="", runtime="", start_file="", private=False, author={}, is_file: bool = False):
+    language="", runtime="", start_file="", private=False, author=None, is_file: bool = False):
         global STARTER_FILES
 
         self._id = _id
@@ -97,6 +100,72 @@ class RunIt:
             return name
         except KeyboardInterrupt:
             sys.exit(1)
+    
+    @staticmethod
+    def notfound(format='json'):
+        global TEMPLATES_FOLDER
+        global NOT_FOUND_FILE
+
+        if format == 'json':
+            return '404 - Not Found'
+
+        with open(os.path.join(os.curdir, TEMPLATES_FOLDER, NOT_FOUND_FILE),'rt') as file:
+            return file.read()
+    
+    @staticmethod
+    def extract_project(filepath):
+        directory = os.path.split(filepath)[0]
+    
+        with ZipFile(filepath, 'r') as file:
+            file.extractall(directory)
+        os.unlink(filepath)
+    
+    def get_functions(self)->list:
+        lang_parser = LanguageParser.detect_language(self.start_file, self.runtime, RunIt.DOCKER, self._id)
+        return lang_parser.list_functions()
+    
+    @staticmethod
+    def dockerize(project_path: str):
+        '''
+        Create docker image for project
+
+        @params project_path
+        @return None
+        '''
+        try:
+            import docker
+
+            client = docker.from_env()
+
+            project_id = os.path.split(project_path)[-1]
+            print(f"[-] Building image for {project_id}")
+
+            image = client.images.build(
+                path=project_path,
+                tag=project_id,
+            )
+            print(image)
+
+        except ImportError:
+            print('[!] Docker package not installed.')
+            print('[-] Use `pip install docker` to install the package.')
+            sys.exit(1)
+        except Exception as e:
+            print(str(e))
+            sys.exit(1)
+    
+    @staticmethod
+    def run_background_task(target: Callable, *args: list):
+        '''
+        Runction for running threads
+
+        @param target Function to run
+        @param args Arguments for the function
+        @return None
+        '''
+        bg_thread = Thread(target=target, args=(args))
+        bg_thread.start()
+        bg_thread.join()
 
     @classmethod
     def is_private(cls, project_id: str, projects_folder: str = PROJECTS_DIR)-> bool:
@@ -129,12 +198,11 @@ class RunIt:
         
         start_file = project.start_file
 
-        lang_parser = LanguageParser.detect_language(start_file, os.getenv('RUNTIME_'+project.language.upper(), project.runtime))
+        lang_parser = LanguageParser.detect_language(start_file, os.getenv('RUNTIME_'+project.language.upper(), project.runtime), RunIt.DOCKER, project_id)
         lang_parser.current_func = func
         try:
             return getattr(lang_parser, func)(*args_list)
         except AttributeError as e:
-            # return f"Function with name '{func}' not defined!"
             return RunIt.notfound()
         except TypeError as e:
             try:
@@ -142,27 +210,7 @@ class RunIt:
             except TypeError as e:
                 return str(e)
     
-    @staticmethod
-    def notfound():
-        global TEMPLATES_FOLDER
-        global NOT_FOUND_FILE
-
-        with open(os.path.join(os.curdir, TEMPLATES_FOLDER, NOT_FOUND_FILE),'rt') as file:
-            return file.read()
-    
-    @staticmethod
-    def extract_project(filepath):
-        directory, filename = os.path.split(filepath)
-    
-        with ZipFile(filepath, 'r') as file:
-            file.extractall(directory)
-        os.unlink(filepath)
-    
-    def get_functions(self)->list:
-        lang_parser = LanguageParser.detect_language(self.start_file, self.runtime)
-        return lang_parser.list_functions()
-    
-    def serve(self, func: str = 'index', args: Optional[Union[dict,list]]=None, filename: str = ''):
+    def serve(self, func: str = 'index', args: Optional[Union[dict,list]]=None):
         global NOT_FOUND_FILE
         
         lang_parser = LanguageParser.detect_language(self.start_file, self.runtime)
@@ -177,7 +225,6 @@ class RunIt:
             return getattr(lang_parser, func)(*args_list)
         except AttributeError as e:
             return RunIt.notfound()
-            # return f"Function with name '{func}' not defined!"
         except TypeError as e:
             try:
                 return getattr(lang_parser, func)()
@@ -227,7 +274,7 @@ class RunIt:
         # os.chdir(CURRENT_PROJECT_DIR)
         zipname = f'{self.name}.zip'
         
-        exclude_list = [zipname, 'account.db', '.git', '.venv', 'venv']
+        exclude_list = [zipname, 'account.db', '.git', '.venv', 'venv', 'Dockerfile']
         
         if '.runitignore' in os.listdir():
             with open('.runitignore', 'rt') as file:
@@ -240,13 +287,12 @@ class RunIt:
 
         with ZipFile(zipname, 'w') as zipobj:
             print('[!] Compressing Project Files...')
-            for folderName, subfolders, filenames in os.walk(os.curdir):
-                
+            for folder_name, subfolders, filenames in os.walk(os.curdir):
                 for filename in filenames:
-                    if not os.path.normpath(os.path.dirname(folderName)).split(os.path.sep)[0] in exclude_list:
-                        filepath = os.path.join(folderName,  filename)
+                    if os.path.normpath(os.path.dirname(folder_name)).split(os.path.sep)[0] not in exclude_list:
+                        filepath = os.path.join(folder_name,  filename)
 
-                        if not os.path.basename(filepath) in exclude_list and not '__pycache__' in folderName:
+                        if os.path.basename(filepath) not in exclude_list and '__pycache__' not in folder_name:
                             zipobj.write(filepath, filepath)
                             print(f'[{filepath}] Compressed!')
             print(f'[!] Filename: {zipname}')
@@ -257,7 +303,7 @@ class RunIt:
         self.config['name'] = self.name
         self.config['version'] = self.version
         self.config['description'] = self.description
-        self.config['homepage'] = 'https://example.com/project_id/'
+        self.config['homepage'] = 'http://example.com/project_id/'
         self.config['language'] = self.language
         self.config['runtime'] = self.runtime
         self.config['private'] = self.private
@@ -273,21 +319,26 @@ class RunIt:
         self.LANGUAGE_TEMPLATE_FILES = os.listdir(self.LANGUAGE_TEMPLATE_FOLDER)
         
         for template_file in self.LANGUAGE_TEMPLATE_FILES:
-            with open(os.path.join(self.LANGUAGE_TEMPLATE_FOLDER,
-                template_file),'rt') as file:
-                with open(os.path.join(os.curdir, self.name, template_file), 'wt') as starter:
-                    starter.write(file.read())
+            if os.path.isfile(os.path.join(self.LANGUAGE_TEMPLATE_FOLDER,template_file)):
+                with open(os.path.join(self.LANGUAGE_TEMPLATE_FOLDER,
+                    template_file),'rt') as file:
+                    with open(os.path.join(os.curdir, 
+                                           self.name, 
+                                           template_file), 'wt') as starter:
+                        starter.write(file.read())
             
         with open(os.path.join(TEMPLATES_FOLDER, NOT_FOUND_FILE),'rt') as file:
-            with open(os.path.join(os.curdir, self.name, NOT_FOUND_FILE), 'wt') as error:
-                error.write(file.read())
+            with open(os.path.join(os.curdir, self.name, NOT_FOUND_FILE), 'wt') as error_404:
+                error_404.write(file.read())
         
         with open(os.path.join(TEMPLATES_FOLDER, DOT_RUNIT_IGNORE),'rt') as file:
             with open(os.path.join(os.curdir, self.name, DOT_RUNIT_IGNORE), 'wt') as dotrunitignore:
                 dotrunitignore.write(file.read())
     
     def install_dependency_packages(self):
-        # os.chdir(os.path.join(os.curdir, self.name))
+        project_path = os.path.realpath(os.path.join(os.curdir, self.name))
+        if project_path != os.path.realpath(os.curdir):
+            os.chdir(project_path)
         packaging_functions = {}
         packaging_functions['javascript'] = self.update_and_install_package_json
         packaging_functions['php'] = self.update_and_install_composer_json
@@ -301,9 +352,9 @@ class RunIt:
             self.update_and_install_package_json()
             self.update_and_install_composer_json()
             self.install_python_packages()
-        except:
+        except Exception:
             print("[!] Couldn't install packages")
-            print("[~] Try again manually later.")
+            print(INSTALL_MODULE_LATER_MESSAGE)
     
     def update_and_install_package_json(self):
         print("[-] Creating and updating package.json")
@@ -323,7 +374,7 @@ class RunIt:
         except Exception as e:
             print(str(e))
             print("[!] Couldn't install modules")
-            print("[~] Try again manually later.")
+            print(INSTALL_MODULE_LATER_MESSAGE)
         
 
     def update_and_install_composer_json(self):
@@ -344,22 +395,25 @@ class RunIt:
         # except Exception as e:
         #     print(str(e))
         #     print("[!] Couldn't install packages")
-        #     print("[~] Try again manually later.")
+        #     print(INSTALL_MODULE_LATER_MESSAGE)
     
     def install_python_packages(self):
-        print("[-] Creating virtual environment")
-        os.system(f"python -m venv venv")
+        print("[-] Creating virtual environment...", end='\r')
+        os.system("python -m venv venv")
+        print("[+] Creating virtual environment - done", end='\n')
         pip_path = os.path.join(os.curdir, 'venv', 'Scripts', 'pip.exe')
         if sys.platform != 'win32':
             pip_path = f".\{os.path.join(os.curdir, 'venv', 'bin', 'pip')}"
         try:
-            print("[-] Installing python packages")
+            print("[-] Installing python packages...", end="\r")
             activate_install_instructions = f"""
             {pip_path} install -r requirements.txt
             """.strip()
             os.system(activate_install_instructions)
+            print("[+] Installing python packages - done", end="\n")
         except Exception as e:
             print(str(e))
             print("[!] Couldn't install packages")
-            print("[~] Try again manually later.")
+            print(INSTALL_MODULE_LATER_MESSAGE)
+    
 
